#include "VoxelBBW.h"

namespace VoxelBBW {

	Mesh * mesh = 0;
	VoxGrid * voxels = 0;
	Handles * handles = 0;

	void openMesh(string meshfile)
	{
		cout << "Loading mesh..."; cout.flush();
		mesh = new Mesh(meshfile);
		cout << "[done]" << endl << endl;
	}						
		
	void computeVoxelGrid(int voxRes)
	{	
		cout << "Voxelizing mesh..." << endl;
		voxels = new VoxGrid(voxRes);
		voxels->initVoxels(*mesh);
		voxels->initStructure();
		cout << "Voxelizing mesh...[done]" << endl << endl;
	}

	void computeBBW(vector<RowVector3> & positions)
	{
		cout << "Creating handles in the voxel grid..."; cout.flush();
		handles = new Handles();
		for(unsigned int i = 0 ; i < positions.size()  ;++i) {
			RowVector3 tp = mesh->transformedPosition(positions[i]);
			RowVector3i node = voxels->getNodeClosestToPoint(tp);
			RowVector3 pos = voxels->getNodePose(node);
			int id = voxels->getNodeId(node);
			handles->addHandle(pos,id);
		}
		cout << "[done]" << endl << endl;
		
		cout << "Computing BBW..." << endl;
		voxels->computeBBW(*handles);
		cout << "Computing BBW...[done]" << endl << endl;

		cout << "Interpolating BBW for mesh vertices..."; cout.flush();
		mesh->computeBBW(*voxels,handles->getNumHandles());
		cout << "[done]" << endl << endl;
	}

	void exportBBW(string meshfile, bool singleFile)
	{
		cout << "Exporting BBW..."; cout.flush();
		string bbwfile = meshfile;
		bbwfile.erase(bbwfile.end()-4,bbwfile.end());
		if(singleFile) {
			bbwfile.append(".bbw");
			mesh->exportBBW(bbwfile,0,handles->getNumHandles()-1);
		}
		else {
			for(int j = 0 ; j < handles->getNumHandles() ; ++j) {
				std::ostringstream file;
				file << bbwfile << j << ".bbw";
				mesh->exportBBW(file.str(),j,j);
			}
		}
		cout << "[done]" << endl;
	}

};