#include "Mesh.h"

#include "VoxGrid.h"

#include "../meshloader/readMesh.h" // basic mesh loader

	Mesh::Mesh(string filename) 
	{
		// read OBJ and OFF file
		readMeshfile(filename,vertices,faces);

		nbV = vertices.rows();
		nbF = faces.rows();
		weights.resize(nbV);

		// compute the transformation between the mesh coordinates and the [0,1]^3 cube
		RowVector3 bmin, bmax;
		bmin = bmax = vertices.row(0);
		
		for(int i=1; i < nbV; i++) {
			bmin = bmin.cwiseMin(vertices.row(i));
			bmax = bmax.cwiseMax(vertices.row(i));    
		}
		
		RowVector3 length = (bmax - bmin).cwiseInverse();
		scale = 0.98*length.minCoeff();
		center = (bmin+bmax)/2.0;
	}
	
	RowVector3 Mesh::transformedPosition(const RowVector3 & p) const 
	{
		return scale*(p-center) + RowVector3(0.5,0.5,0.5);
	}
	RowVector3 Mesh::transformedPosition(int idV) const
	{
		return transformedPosition(vertices.row(idV));
	}
	RowVector3 Mesh::transformedPosition(int idF, int idV) const 
	{
		return transformedPosition(faces(idF,idV));
	}

	void Mesh::computeBBW(const VoxGrid & voxels, int nbWeights)
	{
		for(IndexType i = 0 ; i < nbV ; ++i) {
			voxels.getInterpolatedBBW(transformedPosition(vertices.row(i)),weights[i],nbWeights);
		}
	}

	void Mesh::exportBBW(string filename, int id0, int id1) const
	{	
		ofstream fout;
		fout.open(filename, std::ios::out);
		if(fout.fail()) {
			cout << "I/O error" << endl;
		}
		else {
			for(int i = 0 ; i < nbV ; ++i) {
				for(int j = id0 ; j <= id1 ; ++j) {
					fout <<	weights[i].getCoord(j) << " ";
				}
				fout << endl;
			}
		}
		fout.close();
	}