#ifndef __MESH_H
#define __MESH_H

#include "../STL_inc.h"
#include "../EIGEN_inc.h"
#include "Weights.h"

class VoxGrid;

// simple class to store the mesh information
class Mesh {

private:
	PointMatrixType vertices; // list of vertices (each row is a 3D point)
	FaceMatrixType faces; // list of triangle (each row is a triplet of vertex indices)

	vector<Weights> weights; // list of the weights for each vertex

	// scale and center to transform the initial mesh in a [0,1]^3 cube (needed for the voxelizer)
	ScalarType scale;
	RowVector3 center;

public:
	IndexType nbV; // number of vertices
	IndexType nbF; // number of faces

	Mesh(string filename); // this construction loads list of vertices and triangles from a filename and initialize the data structure
	~Mesh() {}
	
	// returns the position in the voxel grid referential
	RowVector3 transformedPosition(const RowVector3 & p) const;
	RowVector3 transformedPosition(int idV) const;
	RowVector3 transformedPosition(int idF, int idV) const;
	
	// compute the BBW for the ones on the voxel grid using trilinear interpolation
	void computeBBW(const VoxGrid & voxels, int nbWeights);
	
	// export the weights in a file
	void exportBBW(string filename, int id0, int id1) const;

};

#endif