#ifndef __MESHVOXELIZER_H
#define __MESHVOXELIZER_H

#include "slvoxlib.h"
#include "OPENGL_inc.h"
#include "../EIGEN_inc.h"
#include "../core/Mesh.h"
#include "../core/VoxGrid.h"

// the mesh is voxelized using this technique:
// HA-Buffer: Coherent Hashing for single-pass A-buffer http://hal.inria.fr/hal-00811585/
// the library is a courtesy of Sylvain Lefebvre

inline void meshVoxelize(const Mesh & mesh, int res, VoxGrid & voxgrid)
{
	slvox_init(res,2048);

	// begin voxelizing
	slvox_begin();
		
	// draw object to voxelize
	glDisable(GL_LIGHTING);
	glColor4f(1.0f,1.0f,1.0f,1.0f);
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glBegin(GL_TRIANGLES);
	for(int i = 0 ; i < mesh.nbF ; ++i) {
		for(int j = 0 ; j < 3 ; ++j) {
			RowVector3 p = mesh.transformedPosition(i,j);
			glVertex3f(res*p[0],res*p[1],res*p[2]);
		}
	}
	glEnd();
	  
	// done
	slvox_end();
	  
	unsigned char *vox = new unsigned char[res*res*res];
	// collect the result as a binary 3D array and transfer the result to our data structure
	slvox_getvoxels( vox );
	for (int x=0; x<res; x++) {
		for (int y=0; y<res; y++) {
			for (int z=0; z<res; z++) {
				if(vox[res*res*z+res*y+x] != 0) {
					voxgrid.addVoxel(x,y,z);
				}
			}
		}
	}
	delete vox;

	slvox_terminate();
}


#endif